import 'dotenv/config';

import { createLog } from './log';
import type { Logger } from 'winston';
import { readFile } from 'fs/promises';
import { join } from 'path';
import { BskyAgent } from '@atproto/api';
import type CallOptions from '@atproto/api';

import { DB } from './db';
import type { Photo } from './db';
import { envString } from './env';

class Post {
	private log : Logger = createLog('post');
	private db : DB = new DB();
	private bskyAgent : BskyAgent = new BskyAgent({ service: 'https://bsky.social' });

	constructor () {
		this.post();
	}

	private async post () {
		await this.bsky();
	}

	private bskyDescription (photo : Photo) : string {
		return `${photo.description}

${photo.format}${photo.filmstock !== 'Unknown' ? ' - ' + photo.filmstock : ''}`;
	}

	private async bsky () {
		const photo : Photo = await this.db.getBsky();
		const identifier : string = envString('BSKY_USER', null);
		const password : string = envString('BSKY_PASSWORD', null);
		let text : string;
		let post : any;
		let imagePath : string;
		let imageData : Buffer;
		let blobResponse : any;
		let postData : any;

		if (identifier === null || password === null) {
			this.log.error(`Cannot post without Bluesky username or password`);
			return;
		}

		if (photo === null) {
			this.log.warn(`Nothing to post`);
			return;
		}

		try {
			await this.bskyAgent.login({ identifier, password });
			this.log.info(`Logged into Bluesky`);
		} catch (err) {
			this.log.error(`Error logging into Bluesky`, err);
			return;
		}

		text = this.bskyDescription(photo);
/*
                        embed: {
                            images: [
                                {
                                    image: testUpload.data.blob,
                                    alt: "",
                                },
                            ],
                            $type: "app.bsky.embed.images",
                        },
*/
		imagePath = join(envString('WWW', './www'), 'img', `${photo.id}_full.jpg`);

		try {
			imageData = await readFile(imagePath);
		} catch (err) {
			this.log.error(`Error reading image data ${imagePath}`, err);
		}

		try {
			blobResponse = await this.bskyAgent.uploadBlob(imageData, { contentType: 'image/jpeg' } as any)
		} catch (err) {
			this.log.error(`Error uploading blob to Bluesky`, err);
			return;
		}

		postData = {
			text,
			embed: {
			$type: 'app.bsky.embed.images',
			images: [{
					alt: photo.description,
					image: blobResponse.data.blob
				}]
			}
		}

		try {
			await this.bskyAgent.post(postData);
			this.log.info(`Posted ${photo.description} (${photo.name})`);
			console.dir(photo);
		} catch (err) {
			this.log.error(`Error making Bluesky post`, err);
			return;
		}

		try {
			await this.db.confirmBsky(photo.id);
		} catch (err) {
			this.log.error(`Could not confirm that photo ${photo.name} was posted`, err);
			return;
		}

	}
}

new Post();